/*******************************************************************************
*
* File Name: Beschleunigungssensor.c
*
* Beschreibung: main-file fuer das Projekt zum Auslesen eines Beschleinugungsensors
*
* Projekt: Mikrorechnertechnik II 
*
*******************************************************************************/

#define F_CPU 16000000UL

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include "TWI.h"
#include "USART.h"
#include "DATA_SYNC.h"

#define PAUSE						0
#define INITIALISIERUNG				1
#define MESSUNG						2

#define BAUDRATE 9600UL
#define UBRR_BAUDRATE ((F_CPU/(16*BAUDRATE))-1)	//Baudrate im "Asynchronous Normal Mode"
#define BITRATE 100000UL		//(F_CPU/(16+(2*18)*4)) mit TWBR = 18 und TWPS = 1 (TWPS1 und TWPS0 == 0)

/*globale Variable*/
uint8_t  Zustand;
uint8_t DataBuffer=25;


/*******************************************************************************
*Funktion       ISR(USART_RXC_vect)
*
*Beschreibung   Interrupt Service Routine wenn Daten ueber USART-Schnittstelle eingetroffen sind
*
*Parameter		Vektor fuer USART Empfang
*
*Rueckgabe      keine
*
*******************************************************************************/
ISR(USART_RXC_vect)
{
	uint8_t Data;					//Variable fuer die empfangenden Daten
	
	Data=UDR;
	if(DataBuffer<22)				//Wenn zuvor bereits eine Registeradresse eingetroffen ist
	{
		DataArray[0][DataBuffer]=Data;			//angekommenden Daten in das DataArrey an die zuvor uebermittelte Stelle schreiben
		DataArray[1][DataBuffer]=DATA_SYNC_SEND_TO_SENSOR;		//Anmerkung setzen, dass Daten noch an den Sensor gesendet werden muessen
		DataBuffer=25;				//Databuffer auf einen Wert >21 setzen
	}
	else							//Adresse oder Befehl ist angekommen
	{
		switch(Data)
		{
			case 128:				//Befehl in den Zustand Initialisierung zu wechseln
			Zustand=INITIALISIERUNG;
			DATA_SYNC_WriteNewDataToSensor();				//Schreiben der Daten zum Sensor
			DATA_SYNC_ReadAllRegister();					//Auslesen aller Register des Sensors
			DATA_SYNC_WriteNewDataToUsart();				//Schreiben der Registerdaten zur USART
			_delay_ms(3000);
			break;
			
			case 184:				//Befehl in den Zustand Messung zu wechseln
			Zustand=MESSUNG;
			DATA_SYNC_ReadOutputRegister();					//Auslesen des Ausgaberegisters des Sensors
			DATA_SYNC_WriteOutputToUsart();					//Schreiben der Registerdaten zur USART
			_delay_ms(100);
			break;
			
			case 224:				//Befehl zurck in den Zustand Initialisierung zu wechseln
			Zustand=INITIALISIERUNG;
			break;
			
			case 255:				//Befehl in den Zustand Pause zu wechseln
			Zustand=PAUSE;
			_delay_ms(500);
			break;
			
			default:
			if (Data<22)		//Adresse fuer die nachfolgenden Daten
			{
				DataBuffer=Data;
			}
			break;
		}
	}
	
}

/*******************************************************************************
*Funktion       int main(void)
*
*Beschreibung   main Funktion, die zu Beginn des Programms aufgerufen wird
*
*Parameter		keine
*
*Rueckgabe      keine (wird nie verlassen)
*
*******************************************************************************/
int main(void)
{
	/*Initialisierung der Schnittstellen*/
	DATA_SYNC_InitData();
	TWI_InitTWI(18, 0, 0);	
	USART_InitUsart(UBRR_BAUDRATE);

	sei();		//Freischalten der Interrupts		
	
	while (1)
	{
		;
	}
		
}

//int main(void)
//{
	//
	//// Portrichtung
	//DDRD=0xFF;
	//DDRC &= ~((1<<PC5) | (1<<PC4));
	//PORTC|=(1<<PC5) | (1<<PC4);
	//_delay_ms(1000);
	//DDRB=0xFF;
	//
	//// TWI Initialisieren
	//TWBR=32;
	//TWSR=0;
	//
	//TWCR|=(1<<TWINT)|(1<<TWSTA)|(1<<TWEN);    // TWI starten
	//while (!(TWCR & (1<<TWINT)));
	//
	//
	//TWDR=0x70;    // Adresse laden (schreiben)
	//TWCR|=(1<<TWINT)|(1<<TWEN);    // Adresse senden
	//while(!(TWCR&(1<<TWINT)));
	//PORTD=TWSR;
	//
	//TWDR=0x14;    // Zielregister laden
	//TWCR|=(1<<TWINT)|(1<<TWEN);    // Zielreigster ansprechen
	//while(!(TWCR&(1<<TWINT)));
	//
	//
	//TWDR|=(1<<0);    // Inhalt frs Register 0x14h laden
	//TWCR|=(1<<TWINT)|(1<<TWEN);    // Inhalt senden
	//while(!(TWCR&(1<<TWINT)));
	//
	//TWCR|=(1<<TWINT)|(1<<TWSTO)|(1<<TWEN);    // TWI beenden
	//while(!(TWCR&(1<<TWINT)));
	//
	//while(1)
	//{
		//_delay_ms(1000);
		//PORTB|=(1<<PB7);
		//_delay_ms(1000);
		//PORTB&=~(1<<PB7);
	//}
//}