/*******************************************************************************
*
* File Name: USART.c
*
* Beschreibung: Software Funktionen fuer USART
*
* Projekt: Mikrorechnertechnik II 
*
*******************************************************************************/

#include "USART.h"

/*******************************************************************************
*Funktion       void USART_InitUsart(void)
*
*Beschreibung   initialisiert das USART-Modul
*				Interrupt beim Empfangen
*				Baudrate: 9600
*				8 Datenbits
*				1 Stoppbit
*				keine Paritaet
*
*Parameter		keine
*
*Rueckgabe      keine
*
*******************************************************************************/
void USART_InitUsart(unsigned int Baudrate)
{
	UBRRH = (unsigned char)(Baudrate>>8);		
	UBRRL = (unsigned char)Baudrate;				//Festlegen der Baudrate
	
	UCSRB = (1<<RXEN) | (1<<TXEN);					//Freigeben von Empfangen und Senden
	UCSRB = (1<<RXCIE);								//Freigeben des Interrupts zum Empfangen
	
	UCSRC = (1<<URSEL) | (1<<UCSZ1) | (1<<UCSZ0);	//Setzen von 8 Datenbits, kein Parittbit, 1 Stopbit

}

/*******************************************************************************
*Funktion       void USART_SendData(uint8_t Data)
*
*Beschreibung   versendet Daten ueber USART-Schnittstelle
*
*Parameter		8 Bit Daten
*
*Rueckgabe      keine
*
*******************************************************************************/
void USART_SendData(uint8_t Data)
{

	/*Warten, bis der Sendebuffer leer ist*/
	while ( !(UCSRA & (1<<UDRE)) );
	
	/*Daten in den Buffer legen, die gesendet werden sollen*/
	UDR = Data;
}

/************************************************************************/
/* Register der USART:
	- UDR: USART I/O Data Register, 8 Bits zum Senden (RXB), 8 Bits zum Empfangen (TXB)
	- UBRR: USART Baud Rate Register, 12 Bit Register
		- UBRRH: enthlt die 4 hchstwertigen Bits
		- UBRRL: enthlt die 8 niedrigstwertigen Bits
	- UCSRA: USART Control and Status Register A
		- UDRE: USART Data Register Empty, prft, ob das Register leer ist 
	- UCSRB: USART Control and Status Register B
		- RXEN: Receiver Enable, Freischalten der Empfangsfunktion
		- TXEN: Transmitter Enable, Freischalten der Sendefunktion
	- UCSRC: USART Control and Status Register C
		- URSEL: Register Select, whlt zwischen UCSRC und UBRRH Register aus
		- UCSZ: Character Size, legt die Eigenschaften der USART Schnittstelle fest (Paritt, Datenbits, Stopbits)  
*/                                                                   
/************************************************************************/